/*
 Interface.cc - originally 'UIComponent.c'
  
 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/
 
// Interface.cc - base class for all user interface components

#include "Interface.h"
#include <assert.h>
#include <stdio.h>


Interface::Interface ( const char *name ) : Foundation ( name )
{
	// empty
}

void Interface::widgetDestroyedCallback ( Widget, XtPointer clientData, XtPointer )
{
	Interface * obj = ( Interface * ) clientData;
	obj->widgetDestroyed();
}

void Interface::widgetDestroyed()
{ // protected so derived classes can interject additional cleanup
	_w = NULL;
}

void Interface::installDestroyHandler()
{ // called by derived classes after base widget creation
	assert ( _w != NULL );
	XtAddCallback ( _w, XmNdestroyCallback, &Interface::widgetDestroyedCallback,
					( XtPointer ) this );
}

void Interface::manage()
{
	assert ( _w != NULL );
	assert ( XtHasCallbacks ( _w, XmNdestroyCallback ) == XtCallbackHasSome );

	XtManageChild ( _w );
}

Interface::~Interface()
{	// called before Foundation destructor when an object is deleted
	if ( _w )	// check if already destroyed
	{
		// remove destroy callback so it can't be called with ptr to freed obj
		XtRemoveCallback ( _w, XmNdestroyCallback,
						   &Interface::widgetDestroyedCallback,
						   ( XtPointer ) this );
	}
}

void Interface::getResources ( const XtResourceList resources, int numResources )
{
	assert ( _w != NULL );
	assert  ( resources != NULL );

	// retrieve requested resources relative to parent of this obj's
	// base widget
	XtGetSubresources ( XtParent ( _w ), (XtPointer ) this,
			    _name, className(), resources, numResources,
				NULL, 0 );
}

void Interface::setDefaultResources ( const Widget w, const String *resourceSpec )
{
	int i;
	Display *dpy = XtDisplay ( w );	// retrieve display pointer
	XrmDatabase rdb = NULL;		// declare a resource database

	rdb = XrmGetStringDatabase ( "" );	// create empty database
	
	// add this component's resources, prepending component name
	i = 0;
	
	while ( resourceSpec[i] != NULL )
	{
		char buf [ 1000 ]; 	// don't like this at all
		sprintf ( buf, "*%s%s", _name, resourceSpec[i++] );
		XrmPutLineResource ( &rdb, buf );
	}

	// Merge resources into Xt database (with lowest precedence)
	if ( rdb )
	{
		XrmDatabase db = XtDatabase ( dpy );
		XrmCombineDatabase ( rdb, &db, FALSE );
	}
}
